@@ -21,7 +21,8 @@ from traitlets import (
     Int, Long, Integer, Float, Complex, Bytes, Unicode, TraitError,
     Union, All, Undefined, Type, This, Instance, TCPAddress, List, Tuple,
     ObjectName, DottedObjectName, CRegExp, link, directional_link,
-    ForwardDeclaredType, ForwardDeclaredInstance, validate, observe, default
+    ForwardDeclaredType, ForwardDeclaredInstance, validate, observe, default,
+    observe_compat,
 )
 from ipython_genutils import py3compat
 from ipython_genutils.testing.decorators import skipif
@@ -37,7 +38,7 @@ def change_dict(*ordered_values):
 
 class HasTraitsStub(HasTraits):
 
-    def _notify_change(self, name, type, change):
+    def notify_change(self, change):
         self._notify_name = change['name']
         self._notify_old = change['old']
         self._notify_new = change['new']
@@ -556,18 +557,28 @@ class TestObserveDecorator(TestCase):
 
         class A(HasTraits):
             a = Int()
+            b = Int()
             _notify1 = []
+            _notify_any = []
+            
             @observe('a')
             def _a_changed(self, change):
                 self._notify1.append(change)
 
+            @observe(All)
+            def _any_changed(self, change):
+                self._notify_any.append(change)
+
         a = A()
         a.a = 0
-        # This is broken!!!
         self.assertEqual(len(a._notify1),0)
         a.a = 10
         change = change_dict('a', 0, 10, a, 'change')
         self.assertTrue(change in a._notify1)
+        a.b = 1
+        self.assertEqual(len(a._notify_any), 2)
+        change = change_dict('b', 0, 1, a, 'change')
+        self.assertTrue(change in a._notify_any)
 
         class B(A):
             b = Float()
@@ -621,7 +632,7 @@ class TestObserveDecorator(TestCase):
                 super(A, self).__init__(**kwargs)
                 self.observe(self.listener1, ['a'])
             
-            def listener1(self):
+            def listener1(self, change):
                 self.b += 1
 
         class B(A):
@@ -633,11 +644,11 @@ class TestObserveDecorator(TestCase):
                 super(B, self).__init__(**kwargs)
                 self.observe(self.listener2)
             
-            def listener2(self):
+            def listener2(self, change):
                 self.c += 1
             
             @observe('a')
-            def _a_changed(self):
+            def _a_changed(self, change):
                 self.d += 1
 
         b = B()
@@ -1642,6 +1653,28 @@ class TestDirectionalLink(TestCase):
         b.value = 6
         self.assertEqual(a.value, 5)
 
+    def test_tranform(self):
+        """Test transform link."""
+
+        # Create two simple classes with Int traitlets.
+        class A(HasTraits):
+            value = Int()
+        a = A(value=9)
+        b = A(value=8)
+
+        # Conenct the two classes.
+        c = directional_link((a, 'value'), (b, 'value'), lambda x: 2 * x)
+
+        # Make sure the values are correct at the point of linking.
+        self.assertEqual(b.value, 2 * a.value)
+
+        # Change one the value of the source and check that it modifies the target.
+        a.value = 5
+        self.assertEqual(b.value, 10)
+        # Change one the value of the target and check that it has no impact on the source
+        b.value = 6
+        self.assertEqual(a.value, 5)
+
     def test_link_different(self):
         """Verify two traitlets of different types can be linked together using link."""
 
@@ -2024,3 +2057,109 @@ def test_default_value_repr():
     nt.assert_equal(C.n.default_value_repr(), '0')
     nt.assert_equal(C.lis.default_value_repr(), '[]')
     nt.assert_equal(C.d.default_value_repr(), '{}')
+
+
+class TransitionalClass(HasTraits):
+    
+    d = Any()
+    @default('d')
+    def _d_default(self):
+        return TransitionalClass
+
+    parent_super = False
+    calls_super = Integer(0)
+    
+    @default('calls_super')
+    def _calls_super_default(self):
+        return -1
+    
+    @observe('calls_super')
+    @observe_compat
+    def _calls_super_changed(self, change):
+        self.parent_super = change
+    
+    parent_override = False
+    overrides = Integer(0)
+    
+    @observe('overrides')
+    @observe_compat
+    def _overrides_changed(self, change):
+        self.parent_override = change
+
+
+class SubClass(TransitionalClass):
+    def _d_default(self):
+        return SubClass
+    
+    subclass_super = False
+    def _calls_super_changed(self, name, old, new):
+        self.subclass_super = True
+        super(SubClass, self)._calls_super_changed(name, old, new)
+    
+    subclass_override = False
+    def _overrides_changed(self, name, old, new):
+        self.subclass_override = True
+
+
+def test_subclass_compat():
+    obj = SubClass()
+    obj.calls_super = 5
+    nt.assert_true(obj.parent_super)
+    nt.assert_true(obj.subclass_super)
+    obj.overrides = 5
+    nt.assert_true(obj.subclass_override)
+    nt.assert_false(obj.parent_override)
+    nt.assert_is(obj.d, SubClass)
+
+
+class DefinesHandler(HasTraits):
+    parent_called = False
+    
+    trait = Integer()
+    @observe('trait')
+    def handler(self, change):
+        self.parent_called = True
+
+
+class OverridesHandler(DefinesHandler):
+    child_called = False
+    
+    @observe('trait')
+    def handler(self, change):
+        self.child_called = True
+
+
+def test_subclass_override_observer():
+    obj = OverridesHandler()
+    obj.trait = 5
+    nt.assert_true(obj.child_called)
+    nt.assert_false(obj.parent_called)
+
+
+class DoesntRegisterHandler(DefinesHandler):
+    child_called = False
+    
+    def handler(self, change):
+        self.child_called = True
+
+
+def test_subclass_override_not_registered():
+    """Subclass that overrides observer and doesn't re-register unregisters both"""
+    obj = DoesntRegisterHandler()
+    obj.trait = 5
+    nt.assert_false(obj.child_called)
+    nt.assert_false(obj.parent_called)
+
+
+class AddsHandler(DefinesHandler):
+    child_called = False
+    
+    @observe('trait')
+    def child_handler(self, change):
+        self.child_called = True
+
+def test_subclass_add_observer():
+    obj = AddsHandler()
+    obj.trait = 5
+    nt.assert_true(obj.child_called)
+    nt.assert_true(obj.parent_called)
